if (typeof EditTools != "undefined") {
	EditTools.getSuggestions = function() {
	};
}
/**
 * Initializes this auto-completion for the specified textarea object.
 * 
 * The completionFun is optional. If specified, this function will called to
 * create some auto-completion suggestions (instead of ajax-based default
 * behavior). The method gets two parameters, a callback function to be called
 * with the auto-completion suggestions and the prefix to create the suggestions 
 * from. The cannback function must be called with a JSON object representing the desired
 * completion suggestions. The JSON object must be an array of objects with the following
 * attributes each: 
 * <ul>
 * <li>insertText: text to be inserted if item is selected 
 * <li>iconPath: (optional) src-path to the icon of this item 
 * <li>title: (optional) title to be displayed 
 * <li>description: (optional) description text 
 * <li>replaceLength: (optional) number of characters to be replaced before cursor 
 * <li>cursorPosition: (optional) where is the cursor after the completion has been 
 * inserted (relative from start of inserted text)
 * </ul> 
 */
function AutoComplete(area, completionFun) {
	if (area.autocomplete != null) {
		return area.autocomplete;
	}
	this.listElements = null;
	this.completionFun = completionFun;
	this.area = area;
	area.autocomplete = this;
	if (area.textarea == null) {
		new TextArea(area);
	}
	jq$(this.area)
		.click(jq$.proxy(this.handleClickArea, this))
		.keyup(jq$.proxy(this.handleKeyUp, this))
		.keydown(jq$.proxy(this.handleKeyDown, this));
}

AutoComplete.prototype.handleClickArea = function() {
	// preselect value template range (&laquo; to &raquo;) if cursor is
	// inside it
	this.markCompletionParameter();
	// focus goes back to edit field on click inside
	this.setSelectedMenuItem(null);
	this.releaseFocus();
	// an request completions
	this.requestCompletions();
}

// method to prevent scroll-wheel-event
// in auto-completion list to scroll the global document
AutoComplete.prototype.handleScroll = function(e) {
	var f = jq$("#findSuggestionMenu");
	if (f) {
		// check the scroll position
		var maxScroll = f.firstChild.clientHeight - f.clientHeight;
		var curScroll = f.scrollTop;
		// check if the wheel goes up/down
		var delta = 0;
		if (event.wheelDelta) {
			// IE and Opera
			delta = event.wheelDelta / 60;
		} else if (event.detail) {
			// W3C
			delta = -event.detail / 2;
		}
		// stop event if we reached the end of scroll range
		var bottomReached = curScroll >= maxScroll && delta < 0;
		var topReached = curScroll <= 0 && delta > 0;
		if (bottomReached || topReached) {
			jq$.event.fix(e).stopPropagation();
		}
	}
}
AutoComplete.prototype.handleKeyDown = function(event) {
	// preselect value template range (&laquo; to &raquo;) if cursor is
	// inside it
	this.markCompletionParameter();
	event = jq$.event.fix(event);
	if (event.which == 32 && (event.ctrlKey == true || event.altKey == true)) {
		event.stopPropagation();
		event.preventDefault();
		this.requestFocus();
		this.requestCompletions();
		return;
	}
	if (this.hasFocus() && this.menuItems) {
		switch (event.which) {
		case 38: // up
			event.stopPropagation();
			event.preventDefault();
			var item = this.getSelectedMenuItem();
			var next = (item == null) ? null : item.previousSibling;
			if (next == null)
				next = this.menuItems[this.menuItems.length - 1];
			this.setSelectedMenuItem(next);
			return;
		case 40: // down
			event.stopPropagation();
			event.preventDefault();
			var item = this.getSelectedMenuItem();
			var next = (item == null) ? null : item.nextSibling;
			if (next == null)
				next = this.menuItems[0];
			this.setSelectedMenuItem(next);
			return;
		case 39: // right
			// event.stopPropagation();
			return;
		case 37: // left
			// event.stopPropagation();
			return;
		case 27: // escape
			event.stopPropagation();
			event.preventDefault();
			this.releaseFocus();
			this.setSelectedMenuItem(null);
			this.requestCompletions();
			return;
		case 13: // return
			this.releaseFocus();
			event.stopPropagation();
			event.preventDefault();
			this.insertCompletion();
			return;
		default:
			// this.releaseFocus();
			// this.setSelectedMenuItem(null);
			this.requestCompletions();
		}
	}
}
AutoComplete.prototype.handleKeyUp = function() {
	// preselect value template range (&laquo; to &raquo;) if cursor is
	// inside it
	var area = this.area;
	this.markCompletionParameter();
	this.requestCompletions();
}
AutoComplete.prototype.hasFocus = function() {
	return this.focus;
}
AutoComplete.prototype.requestFocus = function() {
	if (this.hasFocus()) return;
	this.focus = true;
	if (this.menuItems) {
		this.setSelectedMenuItem(this.menuItems[0]);
	}
}
AutoComplete.prototype.releaseFocus = function() {
	if (!this.hasFocus()) return;
	this.focus = false;
}
AutoComplete.prototype.getSelectedMenuItem = function() {
	if (!this.menuItems) return null;
	for ( var i = 0; i < this.menuItems.length; i++) {
		var menuItem = this.menuItems[i];
		if (jq$(menuItem).hasClass("hover")) {
			return menuItem;
		}
	}
	return null;
}
AutoComplete.prototype.setSelectedMenuItem = function(element) {
	// adjust tooltip
	if (this.tooltip) {
		jq$(this.tooltip).remove();
		this.tooltip = null;
	}
	if (!this.menuItems) return;
	
	var d = "findSuggestionMenu", f = jq$('#'+d);
    if (f.length === 0) {
        f = jq$(new Element("div", { id : d }));
        jq$("#favorites").before(f);
        f.bind('mousewheel', AutoComplete.handleScroll);
    }
	for ( var i = 0; i < this.menuItems.length; i++) {
		var menuItem = this.menuItems[i];
		if (menuItem == element) {
			jq$(menuItem).addClass("hover");
			this.scrollToVisible(f[0], menuItem);
		} else {
			jq$(menuItem).removeClass("hover");
		}
	}

	if (element && element.completion.description) {
		var tooltipID = d + "_Tooltip";
		jq$('#'+tooltipID).remove();
		var top = f[0].offsetTop + f[0].offsetHeight;
		var maxHeight = jq$(window).height() - top - 10;
		this.tooltip = new Element("div", {
			id : tooltipID,
			style : "top: " + top + "px; max-height: "+maxHeight + "px;"
		});
		f.append(this.tooltip);
		var image = element.completion.iconPath ? "<img src='"
				+ element.completion.iconPath + "'></img> " : "";
		var tipText = "<div class='tooltip_title'>" + image + "<b>"
				+ this.htmlEscape(element.completion.title) + "</b></div>"
				+ element.completion.description;
		jq$(this.tooltip).html(tipText);
	}
}

AutoComplete.prototype.scrollToVisible = function(menu, element) {
	if (element.offsetTop < menu.scrollTop) {
		this.scrollWithoutMousemoveEvents(menu, element.offsetTop);
	} 
	else if (element.offsetTop + element.clientHeight > menu.clientHeight + menu.scrollTop) {
		this.scrollWithoutMousemoveEvents(menu,
				element.offsetTop + element.clientHeight - menu.clientHeight);
	}
}
AutoComplete.prototype.scrollWithoutMousemoveEvents = function(menu, top) {
	this.removeMousemoveEvents();
	menu.scrollTo(0, top);
	this.addModifiedMousemoveEvents();
}
AutoComplete.prototype.removeMousemoveEvents = function() {
	for ( var i = 0; i < this.listElements.length; i++) {
		jq$(this.listElements[i]).unbind("mousemove");
	}
}
AutoComplete.prototype.addModifiedMousemoveEvents = function() {
	if (!this.listElements) return;
	for ( var i = 0; i < this.listElements.length; i++) {
		var self = this;
		var first = this.listElements[i].first;
		first = true;
		jq$(this.listElements[i]).mousemove(function(event) {
			if (!first) {
				self.requestFocus();
				self.setSelectedMenuItem(this);
			}
			first = false;
		});
	}
}

AutoComplete.prototype.markCompletionParameter = function() {
	var area = this.area;
	var text = area.value;
	var textarea = area.textarea;
	var range = textarea.getSelectionCoordinates();
	var start = range.start, end = range.end
	// do not interfere with selections
	if (start != end) return;
	if (start == 0) return;
	
	// check if we are inside laquo and raquo and find boundaries
	start--;
	while (text.charCodeAt(start) != 171) {
		start--;
		if (start < 0) return; // not found
		if (text.charCodeAt(start) == 187) return; // closing one found
		if (text.charCodeAt(start) == 13) return; // line-break found
	}
	while (text.charCodeAt(end) != 187) {
		end++;
		if (end >= text.length) return; // not found
		if (text.charCodeAt(end) == 171) return; // opening one found
		if (text.charCodeAt(end) == 13) return; // line-break found
	}
	end++;
	// now we have found the brackets
	textarea.setSelection(start, end);
}

AutoComplete.prototype.requestCompletions = function() {
	var self = this;
	var area = this.area;
	// to joba's request, we only request if the user has requested focus
	// to re-activate, it should be enough to comment the following if-block
	// out
	if (!this.hasFocus()) {
		this.showCompletions(null);
		return;
	}
	
	// prepare requesting parameters
	var text = area.value;
	var range = area.textarea.getSelectionCoordinates();
	var prefixParam = text.substring(0, range.start); // use completion at

	// prepare callback function for completions to show
	var callback = function(items) {
		if (!items) return;
		// enhance returned items with optional parameters if missing
		for (var i=0; i<items.length; i++) {
			var item = items[i];
			if (typeof item.title === 'undefined') {
				item.title = item.insertText;
			}
			if (typeof item.replaceLength === 'undefined') {
				var l = 0, text = item.insertText;
				while (prefixParam.substr(prefixParam.length-l) == text.substr(0,l)) {
					item.replaceLength = l;
					l++;
				}
			}
			if (typeof item.cursorPosition === 'undefined') {
				item.cursorPosition = item.title.length;
			}
			// decode laquo and raquo characters
			item.insertText = item.insertText.replace(/&#171;/g, "\u00AB");
			item.insertText = item.insertText.replace(/&#187;/g, "\u00BB");
		}
		// avoid non-required updates (interfere with key control)
		var code = AutoComplete.hashCode(items);
		if (self.lastCode == code) return;
		self.lastCode = code;
		// and show items finally
		self.showCompletions(items);
	}
	
	// and finally request the completions
	if (this.completionFun) {
		this.completionFun(callback, prefixParam);
	}
	else {
		AutoComplete.sendCompletionAction(callback, prefixParam);
	}
}

/**
 * Remove items from the specified array of items that matches a specified pattern.
 * All items that matches the pattern for the specified attribute are removed from
 * the completion items.
 * 
 * @param items an array if completion items to be destructively modified
 * @param key the attribute of the completion items to filter for
 * @param pattern a regular expression to be checked against the specified attribute
 */
AutoComplete.removeCompletion = function(items, key, pattern) {
	if (!items) return;
	for (var i=0; i<items.length; i++) {
		if (items[i][key] && items[i][key].toString().match(pattern)) {
			items.splice(i--, 1);
		}
	}
}

/**
 * modifies the specified array of completion items so that all
 * fully quoted insertTexts appear to be unquoted. The insertTexts 
 * are interpreted as term identifiers where-ever appropriate and 
 * being unquoted accordingly.
 * 
 * @param items an array of completion items to unquote
 */  
AutoComplete.unquoteTermIdentifiers = function(items) {
	if (!items) return;
	for (var i=0; i<items.length; i++) {
		items[i].insertText = AutoComplete.unquoteTermIdentifier(items[i].insertText);
	}
}

AutoComplete.unquoteTermIdentifier = function(identifier) {
	var isSpecial = function(c) {return c==' ' || c=='#' || c=='\\' || c=='"' || c=='\t';};
	var text = "", esc = false, quote = false, start = 'next';
	for (var pos = 0; pos < identifier.length; pos++) {
		var c = identifier.charAt(pos);
		// handle escaped character
		if (esc) {
			// check if only special character are quoted: SPACE # \ " , TAB
			// if not, it appears not to be a term identifier
			if (!isSpecial(c)) return identifier;
			text += c;
			start = false;
			esc = false;
		}
		// handle escaping next character
		if (c == '\\' && quote) {
			esc = true;
			continue;
		}
		if (start === 'separator') {
			if (c == '#') {
				text += c;
				// keep start = true for following quote
				// and go to next char
				start = 'next';
				continue;
			}
			// if path element not followed by a '#', this is not a term identifier 
			return identifier;
		} 
		// at the beginning, we can start a quote
		if (c == '"' && start === 'next') {
			quote = true;
			start = false;
			continue;
		}
		// inside quotes, we can find the end of quoting (marks start of new path element)
		if (c == '"' && quote) {
			quote = false;
			start = true;
			continue;
		}
		// check the all special character are quoted: SPACE # \ " , TAB
		// if not, it appears not to be a term identifier
		if (!quote && isSpecial(c)) return identifier;
		text += c;
		start = false;
	}
	// check if we do not have a quote open or in a wrong state
	if (quote || start === 'next') return identifier;
	return text;
}

AutoComplete.termRequiresQuotes = function(term) {
	return term.match(/[\#\s\\"]/) ? true : false;
}

AutoComplete.sendD3webConditionCompletionAction = function(callback, conditionPrefix) {
	AutoComplete.sendCompletionAction(callback, conditionPrefix, '$d3web/condition');
//	AutoComplete.sendCompletionAction(
//			function(items) {
//				AutoComplete.removeCompletion(items, 'insertText', /^@package.*/);
//				callback(items);
//			}, 
//			"%%Rule\n IF (" + conditionPrefix);
}

AutoComplete.sendD3webActionCompletionAction = function(callback, actionPrefix) {
	AutoComplete.sendCompletionAction(callback, actionPrefix, '$d3web/action');
//	AutoComplete.sendCompletionAction(
//			function(items) {
//				AutoComplete.removeCompletion(items, 'insertText', /^@package.*/);
//				AutoComplete.removeCompletion(items, 'insertText', /^THEN.*/);
//				callback(items);
//			}, 
//			"%%Rule\n IF ()\n THEN " + actionPrefix);
}

AutoComplete.sendD3webFormulaCompletionAction = function(callback, formulaPrefix) {
	AutoComplete.sendCompletionAction(callback, formulaPrefix, '$d3web/timeDB/formula');
//	AutoComplete.sendCompletionAction(
//			function(items) {
//				AutoComplete.removeCompletion(items, 'insertText', /^@package.*/);
//				callback(items);
//			}, 
//			"%%Rule\n IF eval(" + formulaPrefix);
}

AutoComplete.sendD3webValueObjectCompletionAction = function(callback, questionPrefix) {
	AutoComplete.sendCompletionAction(callback, questionPrefix, '$d3web/valueObject');
//	if (questionPrefix.charAt(0) != '"') questionPrefix = '"'+questionPrefix;
//	AutoComplete.sendCompletionAction(
//			function(items) {
//				AutoComplete.removeCompletion(items, 'insertText', /^@package.*/);
//				AutoComplete.removeCompletion(items, 'iconPath', /KnowWEExtension\/images\/(?!question|icon_diagnosis)\w+\.gif/);
//				callback(items);
//			}, 
//			"%%Property " + questionPrefix);
}

AutoComplete.sendCompletionAction = function(callback, prefix, scope) {
	if (!scope) scope = "root";
	jq$.ajax( {
		url: 'action/CompletionAction',
		cache: false,
		data: {prefix: prefix, scope: scope, KWiki_Topic : KNOWWE.helper.gup('page')}
	}).success(function(data) {
		callback(eval(data));
	});
}

AutoComplete.hashCode = function(any) {
	// Hashes a string
	var hash = function(string) {
	    var string = string.toString(), hash = 0;
	    for (var i = 0; i < string.length; i++) {
	        hash = (((hash << 5) - hash) + string.charCodeAt(i)) & 0xFFFFFFFF;
	    }
	    return hash;
	};
    // Deep hashes an object
    var object = function(obj) {
        var result = 0;
        for(var property in obj) {
            if(obj.hasOwnProperty(property)) {
                result += hash(property + AutoComplete.hashCode(obj[property]));
            }
        }
        return result;
    };
	// Does a type check on the passed in value and calls the appropriate hash method
	var types = {
	    'string' : hash,
	    'number' : hash,
	    'boolean' : hash,
	    'object' : object
	    // types 'undefined' or 'null' will have a hash of 0
	    // type function is ignored
	};
	var type = typeof any;
	return any != null && types[type] ? types[type](any) + hash(type) : 0;
};

AutoComplete.prototype.showCompletions = function(items) {
	var area = this.area;
	var d = "findSuggestionMenu", f = jq$('#'+d);
    if (f.length === 0) {
        f = jq$(new Element("div", { id : d }));
        jq$("#favorites").before(f);
    }
	var hadFocusBefore = this.hasFocus();
	this.releaseFocus();
	this.menuItems = null;
	// do not show empty completions-list
	if (items == null || items.length == 0) {
		this.setSelectedMenuItem(null);
		// always showing new completions next time
		this.lastCode = null; 
		f.empty();
		return;
	}

	f.html('<div><ul></ul></div>');// "<b>-- Completions --</b><hr>";
	var l = f.find('ul');
	var self = this;
	
	// first check if we have some items with icons
	var hasIcons = false;
	for ( var i = 0; i < items.length; i++) {
		var item = items[i];
		if (item.iconPath) {
			hasIcons = true;
			break;
		}
	}
	
	// then render the items
	this.listElements = new Array(items.length);
	for ( var i = 0; i < items.length; i++) {
		var item = items[i];
		var listElement = new Element("li");
		jq$(listElement)
			.click(function(event) {
				new Event(event).stop();
				self.insertCompletion(area);
			})
			.mousemove(function(event) {
				self.requestFocus();
				self.setSelectedMenuItem(this);
			});
		this.listElements[i] = listElement;
		// store the item, because variable "item" 
		// will be overwritten after each iteration
		listElement.completion = item;
		var html = this.htmlEscape(item.title);
		if (hasIcons) {
			html = "<img src='" 
				+ (item.iconPath ? item.iconPath : "KnowWEExtension/d3web/icon/spacer16.gif") 
				+ "'></img> " +html;
		}
		jq$(listElement).html(html);
		l.append(listElement);
		this.menuItems = l.children();
	}
	if (hadFocusBefore && this.menuItems && this.menuItems.length > 0) {
		this.requestFocus();
	}
}

AutoComplete.prototype.htmlEscape = function(str) {
    return String(str)
            .replace(/&/g, '&amp;')
            .replace(/"/g, '&quot;')
            .replace(/'/g, '&#39;')
            .replace(/</g, '&lt;')
            .replace(/>/g, '&gt;');
}

AutoComplete.prototype.insertCompletion = function() {
	var area = this.area;
	var menuItem = this.getSelectedMenuItem();
	if (menuItem == null) {
		return;
	}
	var completion = menuItem.completion;
	if (typeof EditTools != "undefined") {
		EditTools.store();
	}
	// completions searched after start of selection but also replacing it
	// to the end
	var range = area.textarea.getSelectionCoordinates();
	var end = range.end;
	var start = range.start - completion.replaceLength;
	// check if the text after end matches the last characters
	// of the text to be inserted. If yes, we also replace them
	var overlapLength = completion.insertText.length;
	var text = area.value;
	while (overlapLength > 0) {
		var textBehind = text.substring(range.end, range.end + overlapLength);
		var endOfInsertText = completion.insertText
				.substr(completion.insertText.length - overlapLength);
		if (textBehind.toLowerCase() == endOfInsertText.toLowerCase()) {
			break;
		}
		overlapLength--;
	}
	area.textarea.setSelection(start, end + overlapLength);
	area.textarea.replaceSelection(completion.insertText);
	area.textarea.setSelection(start + completion.cursorPosition);
	// force refresh of completions after inserting the completion
	this.setSelectedMenuItem(null);
	this.requestCompletions();
	this.markCompletionParameter();
	jq$(area).trigger('autosize');
}

jq$(window).load(function() {
	jq$("#editorarea").each(function() {
		new AutoComplete(this)
	});
});
